#!/usr/bin/env python3

# Scenario: staging site which doesn't exist is used as an argument for
# pegasus-plan --staging-site, and a null pointer exception is thrown 
# instead of an error message indicating that a bad staging site was
# used.

import logging
from pathlib import Path
from Pegasus.api import *

logging.basicConfig(level=logging.INFO)

### Site Catalog ###############################################################
WORK_DIR = Path(__file__).parent.resolve()
LOCAL_STORAGE_DIR = str(WORK_DIR / "wf-output")
LOCAL_SCRATCH_DIR = str(WORK_DIR / "wf-scratch")

sc = SiteCatalog()

local = Site("local")
local_storage = Directory(Directory.LOCAL_STORAGE, LOCAL_STORAGE_DIR)
local_storage.add_file_servers(FileServer("file://" + LOCAL_STORAGE_DIR, Operation.ALL))
local_scratch = Directory(Directory.SHARED_SCRATCH, LOCAL_SCRATCH_DIR)
local_scratch.add_file_servers(FileServer("file://" + LOCAL_SCRATCH_DIR, Operation.ALL))
local.add_directories(local_storage, local_scratch)

# pointing to this file so that the planner doesn't raise an exception about invalid key (just for this example)
local.add_pegasus_profile(SSH_PRIVATE_KEY=WORK_DIR / "fake_credential") 

condorpool = Site("condorpool")
condorpool.add_pegasus_profile(style="condor")
condorpool.add_condor_profile(universe="vanilla")

staging = Site("staging")
staging_dir = Directory(Directory.SHARED_SCRATCH, "/home/panorama/public_html")
staging_dir.add_file_servers(
            FileServer("http://10.100.101.107/~panorama/", Operation.GET),
            FileServer("scp://panorama@10.100.101.107/home/panorama/public_html/", Operation.PUT)
        )
staging.add_directories(staging_dir)

sc.add_sites(local, condorpool, staging)
sc.write()

### Replica Catalog ############################################################
rc = ReplicaCatalog()
rc.add_replica(site="local", lfn="test.txt", pfn="http://bad.com/test.txt")
rc.write()

### Transformation Catalog #####################################################
tc = TransformationCatalog()
tc.add_transformations(
            Transformation(
                "ls",
                site="condorpool",
                pfn="/bin/ls",
                is_stageable=False
            )
        )
tc.write()

### Workflow ###################################################################
wf = Workflow("bad-wf")

wf.add_jobs(Job("ls").add_inputs(File("test.txt")))
# minor typo in staging site name, which will lead to java.lang.NullPointerException: null at edu.isi.pegasus.planner.mapper.staging.Hashed.mapToRelativeDirectory(Hashed.java:150)
# which makes it difficult to know that my error was (a typo) pointing to a staging site
# that doesn't exist

wf.plan(
    output_site="local",
    submit=False, 
    sites=["condorpool"],
    staging_sites={"condorpool": "stagingx"}, # <-- ERROR IS HERE, should be "staging" instead of "stagingx"
    force=True,
    **{"pegasus.data.configuration":"nonsharedfs"}
)
